@@ -17,11 +17,8 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- *   msmtp is released under the GPL with the additional exemption that
- *   compiling, linking, and/or using OpenSSL is allowed.
+ *   along with this program; if not, write to the Free Software Foundation,
+ *   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef SMTP_H
@@ -38,13 +35,12 @@
 
 /* SMTP errors */
 
-/* All 'errstr' arguments must point to buffers that are at least
- * 'errstr_bufsize' characters long.
+/*
  * If a function with an 'errstr' argument returns a value != SMTP_EOK,
- * 'errstr' will contain an error message (maybe just "").
+ * '*errstr' either points to an allocates string containing an error 
+ * description or is NULL.
  * If such a function returns SMTP_EOK, 'errstr' will not be changed.
  */
-extern const size_t errstr_bufsize;
 #define SMTP_EOK		0	/* no error */
 #define SMTP_EIO		1	/* Input/output error */
 #define SMTP_EPROTO		2	/* Protocol violation */
@@ -119,11 +115,19 @@ smtp_server_t smtp_new(FILE *debug, int protocol);
  * smtp_connect()
  *
  * Connect to a SMTP server.
+ * If 'server_canonical_name' is not NULL, a pointer to a string containing the
+ * canonical hostname of the server will be stored in '*server_canonical_name',
+ * or NULL if this information is not available.
+ * If 'server_address' is not NULL, a pointer to a string containing the
+ * network address of the server will be stored in '*server_address',
+ * or NULL if this information is not available.
+ * Both strings are allocated.
  * Used error codes: NET_EHOSTNOTFOUND, NET_ESOCKET, NET_ECONNECT
  * Success: NET_EOK
  */
 int smtp_connect(smtp_server_t *srv, const char *host, int port, int timeout,
-	char *errstr);
+	char **server_canonical_name, char **server_address, 
+	char **errstr);
 
 /*
  * smtp_msg_status()
@@ -142,7 +146,7 @@ int smtp_msg_status(list_t *msg);
  * Used error codes: SMTP_EIO, SMTP_EPROTO
  */
 int smtp_get_greeting(smtp_server_t *srv, list_t **errmsg, char **buf, 
-	char *errstr);
+	char **errstr);
     
 /*
  * smtp_init()
@@ -158,7 +162,7 @@ int smtp_get_greeting(smtp_server_t *srv, list_t **errmsg, char **buf,
  * Used error codes: SMTP_EIO, SMTP_EPROTO, SMTP_EINVAL
  */
 int smtp_init(smtp_server_t *srv, const char *ehlo_domain, list_t **msg, 
-	char *errstr);
+	char **errstr);
 
 /*
  * smtp_tls_init()
@@ -169,7 +173,7 @@ int smtp_init(smtp_server_t *srv, const char *ehlo_domain, list_t **msg,
  */
 #ifdef HAVE_SSL
 int smtp_tls_init(smtp_server_t *srv, const char *tls_key_file, 
-	const char *tls_ca_file, const char *tls_trust_file, char *errstr);
+	const char *tls_ca_file, const char *tls_trust_file, char **errstr);
 #endif /* HAVE_SSL */
 
 /*
@@ -186,7 +190,7 @@ int smtp_tls_init(smtp_server_t *srv, const char *tls_key_file,
  * Used error codes: SMTP_EIO, SMTP_EPROTO, SMTP_EINVAL
  */
 #ifdef HAVE_SSL
-int smtp_tls_starttls(smtp_server_t *srv, list_t **error_msg, char *errstr);
+int smtp_tls_starttls(smtp_server_t *srv, list_t **error_msg, char **errstr);
 #endif /* HAVE_SSL */
 
 /*
@@ -203,7 +207,7 @@ int smtp_tls_starttls(smtp_server_t *srv, list_t **error_msg, char *errstr);
  */
 #ifdef HAVE_SSL
 int smtp_tls(smtp_server_t *srv, const char *hostname, int tls_nocertcheck, 
-	tls_cert_info_t *tci, char *errstr);
+	tls_cert_info_t *tci, char **errstr);
 #endif /* HAVE_SSL */
 
 /*
@@ -251,7 +255,7 @@ int smtp_auth(smtp_server_t *srv,
 	const char *auth_mech,
 	char *(*password_callback)(const char *hostname, const char *user),
 	list_t **error_msg,
-	char *errstr);
+	char **errstr);
 
 /* 
  * smtp_envelope()
@@ -275,7 +279,7 @@ int smtp_send_envelope(smtp_server_t *srv,
 	const char *dsn_notify,
 	const char *dsn_return,
 	list_t **error_msg,
-	char *errstr);
+	char **errstr);
 
 /*
  * smtp_send_mail()
@@ -300,7 +304,7 @@ int smtp_send_envelope(smtp_server_t *srv,
  * Used error codes: SMTP_EIO
  */
 int smtp_send_mail(smtp_server_t *srv, FILE *mailf, int keep_bcc, 
-	long *mailsize, char *errstr);
+	long *mailsize, char **errstr);
 
 /*
  * smtp_end_mail()
@@ -311,7 +315,7 @@ int smtp_send_mail(smtp_server_t *srv, FILE *mailf, int keep_bcc,
  * smtp_end_mail_lmtp() instead.
  * Used error codes: SMTP_EIO, SMTP_EUNAVAIL
  */
-int smtp_end_mail(smtp_server_t *srv, list_t **msg, char *errstr);
+int smtp_end_mail(smtp_server_t *srv, list_t **msg, char **errstr);
     
 /*
  * smtp_end_mail_lmtp()
@@ -342,7 +346,7 @@ int smtp_end_mail_lmtp(smtp_server_t *srv,
 	list_t *recipients, 
 	list_t **errstrs,
 	list_t **error_msgs,
-	char *errstr);
+	char **errstr);
 
 /*
  * smtp_etrn()
@@ -352,7 +356,7 @@ int smtp_end_mail_lmtp(smtp_server_t *srv,
  * Used error codes: SMTP_EIO, SMTP_EINVAL, SMTP_EUNAVAIL, SMTP_EPROTO
  */
 int smtp_etrn(smtp_server_t *srv, const char *etrn_argument, 
-	list_t **msg, char *errstr);
+	list_t **msg, char **errstr);
 
 /*
  * smtp_quit()
@@ -361,7 +365,7 @@ int smtp_etrn(smtp_server_t *srv, const char *etrn_argument,
  * Use smtp_close() after this function.
  * Used error codes: SMTP_EIO, SMTP_EPROTO, SMTP_EINVAL
  */
-int smtp_quit(smtp_server_t *srv, char *errstr);
+int smtp_quit(smtp_server_t *srv, char **errstr);
     
 /*
  * smtp_close()
